Imports ThinkGeo.MapSuite.core
Imports System.Collections.ObjectModel
Imports System.IO
Imports System.Drawing.Imaging

Public Class SeamlessGrid
    Inherits GridFeatureLayer
    Public gridnullvalue As Double = 0
    Public grid_opacity As Double = -1
    'we will draw the grid ourselves so that we can get rid of the lines between cells where rounding occurs
    Protected Overrides Sub DrawCore(ByVal canvas As ThinkGeo.MapSuite.Core.GeoCanvas, ByVal labelsInAllLayers As System.Collections.ObjectModel.Collection(Of ThinkGeo.MapSuite.Core.SimpleCandidate))

        Dim g As GridFeatureSource = TryCast(Me.FeatureSource, GridFeatureSource)
        If Not g.IsOpen Then
            Exit Sub
        End If

        Try
            Dim f As Collection(Of Feature) = g.GetFeaturesInsideBoundingBox(canvas.CurrentWorldExtent, ReturningColumnsType.AllColumns)
         
            Dim ext As RectangleShape = canvas.CurrentWorldExtent
            Dim x1 As Double = ext.UpperLeftPoint.X
            Dim x2 As Double = ext.UpperRightPoint.X
            Dim y1 As Double = ext.UpperLeftPoint.Y
            Dim y2 As Double = ext.LowerRightPoint.Y
            Dim w As Integer = canvas.Width
            Dim h As Integer = canvas.Height
            Dim currentScale As Double = ExtentHelper.GetScale(ext, canvas.Width, GeographyUnit.Meter)
            Dim delta As Double = currentScale / canvas.Width
            delta = 0

            Dim bmp As New Bitmap(w, h)
            Dim gr As Graphics = Graphics.FromImage(bmp)
            Dim c As Color
            Dim cellvalue As Double

            Dim k As Integer = f.Count - 1
            Dim icount As Integer = 0
            Dim ft As Feature

            For icount = 0 To k
                ft = f(icount)
                cellvalue = Double.Parse(ft.ColumnValues("cellvalue"))
                c = findcolor(cellvalue)
                If Not c = Color.Transparent Then
                    'slightly quicker than using ft.GetShape.GetBoundingBox
                    Dim p As PolygonShape = ft.GetShape
                    Dim tul As New PointShape(p.OuterRing.Vertices(0).X, p.OuterRing.Vertices(0).Y + CellSize)
                    Dim tlr As New PointShape(p.OuterRing.Vertices(2).X, p.OuterRing.Vertices(2).Y + CellSize)

                    Dim ul As ScreenPointF = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, tul, canvas.Width, canvas.Height)
                    Dim lr As ScreenPointF = ExtentHelper.ToScreenCoordinate(canvas.CurrentWorldExtent, tlr, canvas.Width, canvas.Height)

                    Dim tempr As New RectangleShape(ul.X, lr.Y, lr.X, ul.Y)
                    gr.FillRectangle(New SolidBrush(Color.FromArgb(255, c.R, c.G, c.B)), _
                    New Rectangle(tempr.UpperLeftPoint.X, tempr.UpperLeftPoint.Y, tempr.Width + 1, tempr.Height + 1))
                End If
            Next
            Dim mstream As New MemoryStream()
            gr.Dispose()

            Dim rect As Rectangle = New Rectangle(0, 0, bmp.Width, bmp.Height)
            Dim bmp2 As New Bitmap(w, h)

            If grid_opacity = -1 Then
                setgridopacity()
            End If
            bmp2 = SetImgOpacity(bmp, CSng(grid_opacity / 255))
            bmp2.Save(mstream, System.Drawing.Imaging.ImageFormat.Png)

            Dim img As New GeoImage(mstream)
            Dim center_point As PointShape = canvas.CurrentWorldExtent.GetCenterPoint()
            Dim cx As Integer = w / 2
            Dim cy As Integer = h / 2

            'check if we are printing or viewing on screen
            If TypeOf (canvas) Is PrinterGeoCanvas Then
                'canvas.DrawScreenImage(img, cx, cy, canvas.Width, canvas.Height, DrawingLevel.LevelOne, 0, 0, 0)
                canvas.DrawScreenImageWithoutScaling(img, cx, cy, DrawingLevel.LevelOne, 0, 0, 0)
            Else
                canvas.DrawScreenImageWithoutScaling(img, cx, cy, DrawingLevel.LevelOne, 0, 0, 0)
            End If
            
        Catch ex As Exception
            'couldn't open features for drawing grid
            Debug.Print("couldn't draw grid: " + ex.Message)
        Finally

        End Try
    End Sub

    Private Sub setgridopacity()
        grid_opacity = 200
    End Sub

    Public Shared Function SetImgOpacity(ByVal imgPic As Image, ByVal imgOpac As Single) As Image
        Dim bmpPic As New Bitmap(imgPic.Width, imgPic.Height)
        Dim gfxPic As Graphics = Graphics.FromImage(bmpPic)
        Dim cmxPic As New ColorMatrix()
        cmxPic.Matrix33 = imgOpac

        Dim iaPic As New ImageAttributes()
        iaPic.SetColorKey(Color.Transparent, Color.Transparent)
        iaPic.SetColorMatrix(cmxPic, ColorMatrixFlag.[Default], ColorAdjustType.Bitmap)
        gfxPic.DrawImage(imgPic, New Rectangle(0, 0, bmpPic.Width, bmpPic.Height), 0, 0, imgPic.Width, imgPic.Height, _
        GraphicsUnit.Pixel, iaPic)
        gfxPic.Dispose()

        Return bmpPic
    End Function

    Public Sub New()    
    End Sub

    Public Sub New(ByVal filename As String)
        Me.PathFilename = filename
    End Sub

    Private Function findcolor(ByVal cellvalue As Double) As Color
        If cellvalue = gridnullvalue Then Return (Color.Transparent)
        Dim testvalue As Integer = (cellvalue - 2500) / 3
        If testvalue > 0 And testvalue < 256 Then
            Return Color.FromArgb(255, testvalue, 205, 205)
        Else
            Return Color.Transparent
        End If
    End Function

    Public Function findgridvalue(ByVal x As Double, ByVal y As Double) As Double
        Dim p As New PointShape(x, y)
        'Debug.Print(x.ToString + " : " + y.ToString)
        Try
            If Not Me.IsOpen Then Return -999
            If Me.QueryTools Is Nothing Then Return -999
            Dim f As Collection(Of Feature) = Me.QueryTools.GetFeaturesContaining(p, ReturningColumnsType.AllColumns)
            'Dim f As Collection(Of Feature) = Me.QueryTools.GetFeaturesNearestTo(p, GeographyUnit.Meter, 1, ReturningColumnsType.AllColumns)
            For Each ft In f
                Dim d As Double
                Double.TryParse(ft.ColumnValues("cellvalue"), d)
                Return d
            Next
        Catch

        End Try
        Return -999
    End Function


End Class
